@@ -1,4 +1,5 @@
 use crate::api::{EventHub, Identity, Secret, SecretList, SecretListFilter, SecretVersion, Status};
+use crate::block_store::sync::SyncBlockStore;
 use std::sync::Arc;
 use std::time::Duration;
 
@@ -36,19 +37,34 @@ pub trait SecretsStore: std::fmt::Debug + Send + Sync {
   fn get_version(&self, block_id: &str) -> SecretStoreResult<SecretVersion>;
 }
 
+#[allow(clippy::type_complexity)]
 pub fn open_secrets_store(
   name: &str,
   url: &str,
+  maybe_remote_url: Option<&str>,
   node_id: &str,
   autolock_timeout: Duration,
   event_hub: Arc<dyn EventHub>,
-) -> SecretStoreResult<Arc<dyn SecretsStore>> {
+) -> SecretStoreResult<(Arc<dyn SecretsStore>, Option<Arc<SyncBlockStore>>)> {
   let (scheme, block_store_url) = match url.find('+') {
     Some(idx) => (&url[..idx], &url[idx + 1..]),
     _ => return Err(SecretStoreError::InvalidStoreUrl(url.to_string())),
   };
 
-  let block_store = open_block_store(block_store_url, node_id)?;
+  let mut block_store = open_block_store(block_store_url, node_id)?;
+
+  let sync_block_store = match maybe_remote_url {
+    Some(remote_url) => {
+      let remote = open_block_store(remote_url, node_id)?;
+
+      let sync_block_store = Arc::new(SyncBlockStore::new(block_store, remote));
+
+      block_store = sync_block_store.clone();
+
+      Some(sync_block_store)
+    }
+    _ => None,
+  };
 
   let secrets_store = match scheme {
     "multilane" => Arc::new(multi_lane::MultiLaneSecretsStore::new(
@@ -60,5 +76,5 @@ pub fn open_secrets_store(
     _ => return Err(SecretStoreError::InvalidStoreUrl(url.to_string())),
   };
 
-  Ok(secrets_store)
+  Ok((secrets_store, sync_block_store))
 }
