@@ -1,18 +1,19 @@
 use super::pw_generator::{generate_chars, generate_words};
+use super::synchronizer::Synchronizer;
 use crate::api::{ClipboardProviding, Event, EventData, EventHub, PasswordGeneratorParam, StoreConfig};
 use crate::block_store::StoreError;
-use crate::clipboard::Clipboard;
+use crate::clipboard::{Clipboard, ClipboardCommon};
 use crate::secrets_store::{open_secrets_store, SecretStoreResult, SecretsStore};
 use crate::service::config::{read_config, write_config, Config};
 use crate::service::error::{ServiceError, ServiceResult};
-#[cfg(unix)]
+#[cfg(any(unix, windows))]
 use crate::service::secrets_provider::SecretsProvider;
 use crate::service::{ClipboardControl, TrustlessService};
-use chrono::Utc;
+use chrono::{DateTime, Utc};
 use log::{error, info};
 use rand::{distributions, thread_rng, Rng};
 use std::collections::{HashMap, VecDeque};
-use std::sync::{Arc, RwLock};
+use std::sync::{Arc, Mutex, RwLock};
 use std::time::Duration;
 
 enum ClipboardHolder {
@@ -113,6 +114,7 @@ impl EventHub for LocalEventHub {
 pub struct LocalTrustlessService {
   config: RwLock<Config>,
   opened_stores: RwLock<HashMap<String, Arc<dyn SecretsStore>>>,
+  synchronizers: Mutex<Vec<Synchronizer>>,
   clipboard: RwLock<Arc<ClipboardHolder>>,
   event_hub: Arc<LocalEventHub>,
 }
@@ -124,6 +126,7 @@ impl LocalTrustlessService {
     Ok(LocalTrustlessService {
       config: RwLock::new(config),
       opened_stores: RwLock::new(HashMap::new()),
+      synchronizers: Mutex::new(vec![]),
       clipboard: RwLock::new(Arc::new(ClipboardHolder::Empty)),
       event_hub: Arc::new(LocalEventHub::new(100)),
     })
@@ -173,14 +176,23 @@ impl TrustlessService for LocalTrustlessService {
       .stores
       .get(name)
       .ok_or_else(|| StoreError::StoreNotFound(name.to_string()))?;
-    let store = open_secrets_store(
+    let (store, maybe_sync_block_store) = open_secrets_store(
       name,
       &store_config.store_url,
+      store_config.remote_url.as_deref(),
       &store_config.client_id,
       Duration::from_secs(store_config.autolock_timeout_secs),
       self.event_hub.clone(),
     )?;
 
+    if let Some(sync_block_store) = maybe_sync_block_store {
+      self.synchronizers.lock()?.push(Synchronizer::new(
+        store.clone(),
+        sync_block_store,
+        chrono::Duration::seconds(store_config.sync_interval_sec as i64),
+      ));
+    }
+
     opened_stores.insert(name.to_string(), store.clone());
 
     Ok(store)
@@ -210,9 +222,8 @@ impl TrustlessService for LocalTrustlessService {
     store_name: &str,
     block_id: &str,
     properties: &[&str],
-    display_name: &str,
   ) -> ServiceResult<Arc<dyn ClipboardControl>> {
-    #[cfg(unix)]
+    #[cfg(any(unix, windows))]
     {
       let store = self.open_store(store_name)?;
       let secret_version = store.get_version(block_id)?;
@@ -225,7 +236,6 @@ impl TrustlessService for LocalTrustlessService {
       info!("Providing {} for {} in {}", properties.join(","), block_id, store_name);
 
       let next_clipboard = Arc::new(ClipboardHolder::Providing(Clipboard::new(
-        display_name,
         secret_provider,
         self.event_hub.clone(),
       )?));
@@ -233,7 +243,7 @@ impl TrustlessService for LocalTrustlessService {
 
       Ok(next_clipboard)
     }
-    #[cfg(not(unix))]
+    #[cfg(not(any(unix, windows)))]
     {
       Err(ServiceError::NotAvailable)
     }
@@ -290,6 +300,41 @@ impl TrustlessService for LocalTrustlessService {
       }
     }
   }
+
+  fn needs_synchronization(&self) -> bool {
+    if let Ok(config) = self.config.read() {
+      config
+        .stores
+        .iter()
+        .any(|(_, store_config)| store_config.remote_url.is_some())
+    } else {
+      false
+    }
+  }
+
+  fn synchronize(&self) -> Option<DateTime<Utc>> {
+    match self.synchronizers.lock() {
+      Ok(mut synchronizers) => {
+        let mut result = None;
+        for synchronizer in synchronizers.iter_mut() {
+          if let Err(err) = synchronizer.synchronize() {
+            error!("Synchronization failed: {}", err);
+          }
+          let next = synchronizer.next_run();
+          result = match result {
+            Some(prev) if prev > next => Some(next),
+            Some(prev) => Some(prev),
+            None => Some(next),
+          };
+        }
+        result
+      }
+      Err(err) => {
+        error!("Synchronization lock failed: {}", err);
+        None
+      }
+    }
+  }
 }
 
 impl std::fmt::Debug for LocalTrustlessService {
