@@ -1,3 +1,5 @@
+use chrono::{DateTime, Utc};
+
 use crate::api::{ClipboardProviding, Event, PasswordGeneratorParam, StoreConfig};
 use std::sync::Arc;
 
@@ -7,9 +9,12 @@ pub mod local;
 pub mod pw_generator;
 mod remote;
 pub mod secrets_provider;
+mod synchronizer;
 
 #[cfg(unix)]
 pub mod unix;
+#[cfg(windows)]
+pub mod windows;
 
 pub use self::config::config_file;
 pub use self::error::*;
@@ -52,7 +57,6 @@ pub trait TrustlessService: std::fmt::Debug + Send + Sync {
     store_name: &str,
     block_id: &str,
     properties: &[&str],
-    display_name: &str,
   ) -> ServiceResult<Arc<dyn ClipboardControl>>;
 
   fn poll_events(&self, last_id: u64) -> ServiceResult<Vec<Event>>;
@@ -62,6 +66,10 @@ pub trait TrustlessService: std::fmt::Debug + Send + Sync {
   fn generate_password(&self, param: PasswordGeneratorParam) -> ServiceResult<String>;
 
   fn check_autolock(&self);
+
+  fn needs_synchronization(&self) -> bool;
+
+  fn synchronize(&self) -> Option<DateTime<Utc>>;
 }
 
 pub fn create_service() -> ServiceResult<Arc<dyn TrustlessService>> {
@@ -71,5 +79,11 @@ pub fn create_service() -> ServiceResult<Arc<dyn TrustlessService>> {
       return Ok(Arc::new(remote));
     }
   }
+  #[cfg(windows)]
+  {
+    if let Some(remote) = self::windows::try_remote_service()? {
+      return Ok(Arc::new(remote));
+    }
+  }
   Ok(Arc::new(self::local::LocalTrustlessService::new()?))
 }
